#include <stdio.h>
#include <windows.h>
#include "../Framework2/Imaging/Image.h"

bool runApp(char *cmdLine){
	STARTUPINFO sInfo;
	PROCESS_INFORMATION pInfo;

	memset(&sInfo, 0, sizeof(sInfo));
	sInfo.cb = sizeof(sInfo);

	if (CreateProcess(NULL, cmdLine, NULL, NULL, TRUE, 0, NULL, NULL, &sInfo, &pInfo) == 0){
		printf("Couldn't run \"%s\"\n", cmdLine);
		return false;
	}

	WaitForSingleObject(pInfo.hProcess, INFINITE);
	return true;
}

int main(int argc, char *argv[]){
	if (argc < 2){
		printf("Usage:\n3DcGenerator <file>\n\n<file> can be of either DDS, TGA, PNG, JPG, PCX or BMP format.\n");
		return -1;
	}

	Image img;
	if (!img.loadImage(argv[1])){
		printf("Couldn't open \"%s\"\n");
		return -1;
	}
	img.toGrayScale();
	img.toNormalMap();

	unsigned char *pixels = img.getImage();
	int nPixels = img.getNumberOfPixels(0, 1);
	int nChannels = getChannels(img.getFormat());

	unsigned char *newPixels = new unsigned char[nPixels * 4];
	memset(newPixels, 0xFF, nPixels * 4);

	char *names[] = { "TempChannelX.dds", "TempChannelY.dds" };
	char str[512];
	int i, j;
	for (j = 0; j < 2; j++){
		for (i = 0; i < nPixels; i++){
			newPixels[4 * i + 3] = pixels[i * nChannels + j];
		}
		Image newImg;
		newImg.loadFromMemory(newPixels, img.getWidth(), img.getHeight(), FORMAT_RGBA8, false, false);
		newImg.saveImage(names[j]);

		sprintf(str, "texconv.exe -f DXT5 -mf BOX -nologo %s", names[j]);
		if (!runApp(str)) return -1;
	}
	delete newPixels;

	Image imgX, imgY;

	imgX.loadImage(names[0], false, MIPMAP_ALLOCATE);
	imgY.loadImage(names[1], false, MIPMAP_ALLOCATE);

	printf("\nGenerating 3Dc texture\n");

	pixels = new unsigned char[imgX.getMipMappedSize()];
	unsigned char *dest = pixels;
	for (j = 0; j < imgX.getNumberOfMipMaps(); j++){
		unsigned char *src0 = imgY.getImage(j);
		unsigned char *src1 = imgX.getImage(j);

		int w = max(imgX.getWidth(j)  / 4, 1);
		int h = max(imgX.getHeight(j) / 4, 1);

		for (i = 0; i < w * h; i++){
			memcpy(dest,     src0, 8);
			memcpy(dest + 8, src1, 8);
			dest += 16;
			src0 += 16;
			src1 += 16;
		}
	}

	printf("Deleting temporary files\n", str);
	
	DeleteFile(names[0]);
	DeleteFile(names[1]);

	char *ext = strrchr(argv[1], '.');
	if (ext) *ext = '\0';
	sprintf(str, "%s3Dc.dds", argv[1]);
	printf("Outputting file: %s\n", str);

	Image outImg;
	outImg.loadFromMemory(pixels, imgX.getWidth(), imgX.getHeight(), FORMAT_3DC, true, true);
	outImg.saveImage(str);

	return 0;
}
